/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('DeploymentFlow', function () {
  var acl;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-flow', function () {
      done();
    });
  });

  beforeEach(function () {
    acl = { isReadOnly: sinon.stub().returns(false) };
  });

  it('can render', function () {
    var addTemplate = sinon.stub();
    var changesFilterByParent = sinon.stub();
    var changeState = sinon.stub();
    var generateAllChangeDescriptions = sinon.stub();
    var listBudgets = sinon.stub();
    var listClouds = sinon.stub();
    var listPlansForCharm = sinon.stub();
    var listRegions = sinon.stub();
    var listTemplates = sinon.stub();
    var servicesGetById = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: addTemplate,
        changesFilterByParent: changesFilterByParent,
        changeState: changeState,
        groupedChanges: {},
        generateAllChangeDescriptions: generateAllChangeDescriptions,
        listBudgets: listBudgets,
        listClouds: listClouds,
        listPlansForCharm: listPlansForCharm,
        listRegions: listRegions,
        listTemplates: listTemplates,
        servicesGetById: servicesGetById,
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      juju.components.Panel,
      {
        instanceName: 'deployment-flow-panel',
        visible: true },
      React.createElement(
        'div',
        { className: 'deployment-flow' },
        React.createElement(
          'div',
          { className: 'deployment-flow__header' },
          React.createElement(
            'div',
            { className: 'deployment-flow__close' },
            React.createElement(juju.components.GenericButton, {
              action: instance._handleClose,
              type: 'neutral',
              title: 'Back to canvas' })
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-flow__content' },
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'div',
              { className: 'inner-wrapper' },
              React.createElement(
                juju.components.DeploymentSection,
                {
                  buttons: undefined,
                  completed: false,
                  disabled: false,
                  instance: 'deployment-cloud',
                  showCheck: true,
                  title: 'Choose cloud to deploy to' },
                React.createElement(juju.components.DeploymentCloud, {
                  acl: acl,
                  cloud: null,
                  clouds: instance.CLOUDS,
                  listClouds: listClouds,
                  setCloud: instance._setCloud })
              ),
              React.createElement(
                juju.components.DeploymentSection,
                {
                  completed: false,
                  disabled: true,
                  instance: 'deployment-credential',
                  showCheck: false },
                React.createElement(juju.components.DeploymentCredential, {
                  acl: acl,
                  addTemplate: addTemplate,
                  credential: null,
                  cloud: null,
                  clouds: instance.CLOUDS,
                  listRegions: listRegions,
                  listTemplates: listTemplates,
                  region: null,
                  setCredential: instance._setCredential,
                  setRegion: instance._setRegion,
                  setTemplate: instance._setTemplate,
                  template: null,
                  users: {},
                  validateForm: instance._validateForm })
              ),
              React.createElement(
                juju.components.DeploymentSection,
                {
                  completed: false,
                  disabled: true,
                  instance: 'deployment-machines',
                  showCheck: false,
                  title: 'Machines to be deployed' },
                React.createElement(juju.components.DeploymentMachines, {
                  acl: acl,
                  cloud: null })
              ),
              React.createElement(
                juju.components.DeploymentSection,
                {
                  completed: false,
                  disabled: true,
                  instance: 'deployment-services',
                  showCheck: true,
                  title: React.createElement(
                    'span',
                    { className: 'deployment-flow__service-title' },
                    'Services to be deployed',
                    React.createElement(juju.components.GenericButton, {
                      action: instance._toggleChangelogs,
                      type: 'base',
                      title: 'Show changelog' })
                  ) },
                React.createElement(juju.components.DeploymentServices, {
                  acl: acl,
                  changesFilterByParent: changesFilterByParent,
                  generateAllChangeDescriptions: generateAllChangeDescriptions,
                  groupedChanges: {},
                  cloud: null,
                  listPlansForCharm: listPlansForCharm,
                  servicesGetById: servicesGetById,
                  showChangelogs: false })
              ),
              React.createElement(
                juju.components.DeploymentSection,
                {
                  completed: false,
                  disabled: true,
                  instance: 'deployment-budget',
                  showCheck: true,
                  title: 'Confirm budget' },
                React.createElement(juju.components.DeploymentBudget, {
                  acl: acl,
                  listBudgets: listBudgets,
                  user: {} })
              ),
              React.createElement(
                'div',
                { className: 'twelve-col' },
                React.createElement(
                  'div',
                  { className: 'deployment-flow__deploy' },
                  React.createElement(
                    'div',
                    null,
                    React.createElement(
                      'div',
                      { className: 'deployment-flow__deploy-option' },
                      React.createElement('input', { className: 'deployment-flow__deploy-checkbox',
                        disabled: false,
                        id: 'emails',
                        type: 'checkbox' }),
                      React.createElement(
                        'label',
                        { className: 'deployment-flow__deploy-label',
                          htmlFor: 'emails' },
                        'Please email me updates regarding feature announcements, performance suggestions, feedback surveys and special offers.'
                      )
                    ),
                    React.createElement(
                      'div',
                      { className: 'deployment-flow__deploy-option' },
                      React.createElement('input', { className: 'deployment-flow__deploy-checkbox',
                        disabled: false,
                        id: 'terms',
                        type: 'checkbox' }),
                      React.createElement(
                        'label',
                        { className: 'deployment-flow__deploy-label',
                          htmlFor: 'terms' },
                        'I agree that my use of any services and related APIs is subject to my compliance with the applicable ',
                        React.createElement(
                          'a',
                          { href: '', target: '_blank' },
                          'Terms of service'
                        ),
                        '.'
                      )
                    )
                  ),
                  React.createElement(
                    'div',
                    { className: 'deployment-flow__deploy-action' },
                    React.createElement(juju.components.GenericButton, {
                      action: undefined,
                      disabled: true,
                      type: 'positive',
                      title: 'Deploy' })
                  )
                )
              )
            )
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can close', function () {
    var changeState = sinon.stub();
    var output = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changesFilterByParent: sinon.stub(),
        changeState: changeState,
        generateAllChangeDescriptions: sinon.stub(),
        groupedChanges: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ));
    output.props.children.props.children[0].props.children.props.children.props.action();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: null,
        metadata: {}
      }
    });
  });

  it('can display the cloud section as complete', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    instance._setCredential('cred');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isTrue(sections[0].props.completed);
  });

  it('correctly sets the cloud title if no cloud is chosen', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.equal(sections[0].props.title, 'Choose cloud to deploy to');
  });

  it('correctly sets the cloud title if a public cloud is chosen', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('azure');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.equal(sections[0].props.title, 'Public cloud');
  });

  it('correctly sets the cloud title if local is chosen', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('local');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.equal(sections[0].props.title, 'Local cloud');
  });

  it('can clear the cloud and credential when changing clouds', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('local');
    instance._setCredential('local');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isNotNull(instance.state.cloud);
    assert.isNotNull(instance.state.credential);
    sections[0].props.buttons[0].action();
    assert.isNull(instance.state.cloud);
    assert.isNull(instance.state.credential);
  });

  it('can enable the credential section', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isFalse(sections[1].props.disabled);
  });

  it('can enable the machines section', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    instance._setCredential('cred');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isFalse(sections[2].props.disabled);
  });

  it('can enable the services section', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    instance._setCredential('cred');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isFalse(sections[3].props.disabled);
  });

  it('can enable the budget section', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    instance._setCredential('cred');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isFalse(sections[4].props.disabled);
  });

  it('can hide the agreements section', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentFlow,
      {
        acl: acl,
        addTemplate: sinon.stub(),
        changeState: sinon.stub(),
        changes: {},
        listBudgets: sinon.stub(),
        listClouds: sinon.stub(),
        listPlansForCharm: sinon.stub(),
        listRegions: sinon.stub(),
        listTemplates: sinon.stub(),
        modelCommitted: true,
        servicesGetById: sinon.stub(),
        user: {},
        users: {} },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    instance._setCloud('cloud');
    instance._setCredential('cred');
    var output = renderer.getRenderOutput();
    var sections = output.props.children.props.children[1].props.children.props.children.props.children;
    assert.isUndefined(sections[5].props.children.props.children[0]);
  });
});